#ifndef SBUF_H_
#define SBUF_H_

#include "raster.h"


//
// siloette hsr extentions
//

namespace raster
{
 struct texture;
 struct solid_surface_desc;


 //
 // a 2d screen space vert
 // with z information
 //
 struct vert_3
 {
    float c[2]; // 2d position
    float z_inv;  // -1.0 / z 

    inline const float& operator [] (int i )const  { return c[i]; }
    inline       float& operator [] (int i )       { return c[i]; }
 };

  //
  // 2d vert with z information
  //             and shade

  struct vert_3s : vert_3 
   {
     float shade;
   };

   //
   // handle of poly, for use in submitting edges
   // 
   struct poly;



  
   extern poly * __fastcall submit_alt(const vert_3s & v0, // first tree verts
                            const vert_3s & v1,
                            const vert_3s & v2,
                            const texture * tex,
                            const float  uv_0[2],  // these are uv pairs for the first 3 verts
                            const float  uv_1[2],
                            const float  uv_2[2]  );

   extern poly * __fastcall submit_plt(const vert_3s & v0,
                            const vert_3s & v1,
                            const vert_3s & v2,
                            const texture * tex,
                            const float  uv_0[2],
                            const float  uv_1[2],
                            const float  uv_2[2]  );

   //
   // note: primitives with transparency require special handling
   //
   extern void __fastcall submit_alt_tp(const vert_3s & v0, // first tree verts
                             const vert_3s & v1,
                             const vert_3s & v2,
                             const texture * tex,
                             const float  uv_0[2],  // these are uv pairs for the first 3 verts
                             const float  uv_1[2],
                             const float  uv_2[2],
                     							 const vert_3 ** all_verts,  // all verts in poly.  Poly must be 
							                                                  // horizontally convex.
                                                         // only c[0], c[1] are used
							                      int num_verts);            // number of verts in poly


   extern void __fastcall submit_aft_tp(
                             const vert_3 & v0, // first tree verts
                             const vert_3 & v1,
                             const vert_3 & v2,
                             const float shade,
                             const texture * tex,
                             const float  uv_0[2],  // these are uv pairs for the first 3 verts
                             const float  uv_1[2],
                             const float  uv_2[2],
                     							 const vert_3 ** all_verts,  // all verts in poly.  Poly must be 
							                                                 // horizontally convex.
                                                        // only c[0], c[1] are used
							                      int num_verts);            // number of verts in poly


   extern void __fastcall submit_plt_tp(const vert_3s & v0,
                            const vert_3s & v1,
                            const vert_3s & v2,
                            const texture * tex,
                            const float  uv_0[2],
                            const float  uv_1[2],
                            const float  uv_2[2],
 						                     const vert_3 ** all_vertss,  // all verts in poly.  Poly must be 
							                            // horizontally convex.
                                   // only c[0], c[1] are used
							                     int num_verts);            // number of verts in poly


   extern void __fastcall submit_pft_tp(
                            const vert_3 & v0,
                            const vert_3 & v1,
                            const vert_3 & v2,
                            const float shade,
                            const texture * tex,
                            const float  uv_0[2],
                            const float  uv_1[2],
                            const float  uv_2[2],
 						                     const vert_3 ** all_vertss,  // all verts in poly.  Poly must be 
							                            // horizontally convex.
                                   // only c[0], c[1] are used
							                     int num_verts);            // number of verts in poly



   extern  poly * __fastcall submit_als(const vert_3s & v0,
                             const vert_3s & v1,
                             const vert_3s & v2,
                             const solid_surface_desc * sd );

   extern  poly * __fastcall submit_pls(const vert_3s & v0,
                             const vert_3s & v1,
                             const vert_3s & v2,
                             const solid_surface_desc * sd );
 

 
  extern poly  * __fastcall submit_aft( const vert_3 & v0,
                             const vert_3 & v1,
                             const vert_3 & v2,
                             float shade,
                             const texture * tex,
                             const float  uv_0[2],
                             const float  uv_1[2],
                             const float  uv_2[2]);

  extern poly  * __fastcall submit_pft( const vert_3 & v0,
                             const vert_3 & v1,
                             const vert_3 & v2,
                             float shade,
                             const texture * tex,
                             const float  uv_0[2],
                             const float  uv_1[2],
                             const float  uv_2[2]);


  extern poly *  __fastcall submit_afs(const vert_3 & v0,
                            const vert_3 & v1,
                            const vert_3 & v2,
                            float shade, 
                            const solid_surface_desc * sd );



  //
  // note: either p0 or p1 must not be  null
  //              p0 and p1 can both not be null
  //
  extern  void __fastcall submit_edge(float x0, float y0, float x1, float y1,poly * p0,poly * p1) ;

  //
  // point is flat shaded.  if width == height == 1.0f, only 1 pixel is drawn
  //
  extern void __fastcall submit_rect(const vert_3s & c, 
                           float width, // thickness at vert, in pixels
                           float height, // thickness at second vert, in pixels,
                           const solid_surface_desc * sd );

  //
  // line is linearly shaded by default
  // is iv0.shade == iv1.shade, line is flat shaded,
  // and is rendered slightly faster
  //
  extern void __fastcall submit_line(const vert_3s & iv0, 
                          const vert_3s & iv1,
                          float thick0, // thickness at vert, in pixels
                          float thick1, // thickness at second vert, in pixels,
                          const solid_surface_desc * sd );


  
  //
  // call after submitting a batch of polys
  //
  extern  void __fastcall sweep_and_draw();

  //
  // call when all other drawing is complete.
  // draws transparent polys back to front
  //
  extern  void __fastcall draw_transparent_polys();


 };
#endif